% A New Keynesian model in a small open economy
% Replicates the model from 'On fiscal multipliers in New Keynesian small 
%   open economy models' by Philip Schuster
% Philip Schuster May 28, 2020, philip.schuster@oenb.at

close all;

% INSTRUCTION: run simulation via runNK.m


%================================================================
% 1.a Control Center (user interface)
%================================================================

%%%%%%%%%%%%%% shock_size %%%%%%%%%%%%%%%
%----------------------------------------
shock_size      = 0.001;
%----------------------------------------

%%%%%%%%%%%%%%% shocktype %%%%%%%%%%%%%%%
%%% 1.. deter & permanent 2.. deter & temporary, 3.. stoch & temporary
%----------------------------------------
@#define shocktype = 1
%----------------------------------------

%%%%%%%%%%%%%%% instrument %%%%%%%%%%%%%%
%%% 1.. CG,   2.. IG,      3.. subI, 4..  subL, 5..  tauC, 6.. tauW,
%%% 7.. tauF, 8.. tauprof, 9.. tauK, 10.. tauR, 11.. tauLC
%----------------------------------------
@#define instrument = 1
%----------------------------------------

%%%%%% finite_epsilon (default = 1) %%%%%
%%% 1.. epsilon is finite, 0.. epsilon is infinite
%%% Note: if finite_epsilon == 0, epsilon should be set to Inf in
%%%       runNK.m (for accurate steady state guess)
%----------------------------------------
@#define finite_epsilon = 1
%----------------------------------------

%%%%% finite_epsilonW (default = 1) %%%%%
%%% 1.. epsilonW is finite, 0.. epsilonW is infinite
%%% Note: if finite_epsilonW == 0, epsilonW should be set to Inf in
%%%       runNK.m (for accurate steady state guess)
%----------------------------------------
@#define finite_epsilonW = 1
%----------------------------------------

%%%%%% finite_delta_2 (default = 1) %%%%%
%%% 1.. delta_2 is finite, 0.. delta_2 is infinite
%%% Note: if finite_delta_2 == 0, delta_2 should be set to Inf in
%%%       runNK.m (for accurate steady state guess)
%----------------------------------------
@#define finite_delta_2 = 1
%----------------------------------------

%%%%%%% no_constHH (default = 0) %%%%%%%%
%%% 1.. no constrained HH (unconst set to 1.0), 0.. some constrained HH
%%% Note: if no_constHH == 0, unconst should be set to 1.0 in
%%%       runNK.m (for accurate steady state guess)
%----------------------------------------
@#define no_constHH = 0
%----------------------------------------

%%%%%%%%% CD_prod (default = 1) %%%%%%%%%
%%% 1.. Cobb-Douglas production(sigmaP ignored), 0.. CES
%%% Note: if CD_prod == 0, sigmaP must not be equal to 1.0 in runNK.m
%%% Note: if CD_prod == 1, sigmaP should be set to 1.0 in runNK.m
%%%       (for accurate steady state guess)
%----------------------------------------
@#define CD_prod = 1
%----------------------------------------

%%%%%%%%% CD_Cons (default = 0) %%%%%%%%%
%%% 1.. Cobb-Douglas aggregator Cons (sigCm ignored), 0.. CES
%%% Note: if CD_Cons == 0, sigCm must not be equal to 1.0 in runNK.m
%%% Note: if CD_Cons == 1, sigCm should be set to 1.0 in runNK.m
%%%       (for accurate steady state guess)
%----------------------------------------
@#define CD_Cons = 0
%----------------------------------------

%%%%%%%%% CD_CG (default = 0) %%%%%%%%%%%
%%% 1.. Cobb-Douglas aggregator CG (sigGm ignored), 0.. CES
%%% Note: if CD_CG == 0, sigGm must not be equal to 1.0 in runNK.m
%%% Note: if CD_CG == 1, sigGm should be set to 1.0 in runNK.m
%%%       (for accurate steady state guess)
%----------------------------------------
@#define CD_CG = 0
%----------------------------------------

%%%%%%%%% CD_IG (default = 0) %%%%%%%%%%%
%%% 1.. Cobb-Douglas aggregator IG (sigIGm ignored), 0.. CES
%%% Note: if CD_IG == 0, sigIGm must not be equal to 1.0 in runNK.m
%%% Note: if CD_IG == 1, sigIGm should be set to 1.0 in runNK.m
%%%       (for accurate steady state guess)
%----------------------------------------
@#define CD_IG = 0
%----------------------------------------

%%%%%%%%% CD_Inv (default = 0) %%%%%%%%%%
%%% 1.. Cobb-Douglas aggregator Inv (sigIm ignored), 0.. CES
%%% Note: if CD_Inv == 0, sigIm must not be equal to 1.0 in runNK.m
%%% Note: if CD_Inv == 1, sigIm should be set to 1.0 in runNK.m
%%%       (for accurate steady state guess)
%----------------------------------------
@#define CD_Inv = 0
%----------------------------------------

%================================================================
% 1.b Control Center (preprocessing)
%================================================================

@#if (shocktype == 1)
    shocktype = 1;
@#endif
@#if (shocktype == 2)
    shocktype = 2;
@#endif
@#if (shocktype == 3)
    shocktype = 3;
@#endif

@#if (instrument == 1)
    instrumentname = 'CG';
@#endif
@#if (instrument == 2)
    instrumentname = 'IG';
@#endif
@#if (instrument == 3)
    instrumentname = 'subI';
@#endif
@#if (instrument == 4)
    instrumentname = 'subL';
@#endif
@#if (instrument == 5)
    instrumentname = 'tauC';
@#endif
@#if (instrument == 6)
    instrumentname = 'tauW';
@#endif
@#if (instrument == 7)
    instrumentname = 'tauF';
@#endif
@#if (instrument == 8)
    instrumentname = 'tauprof';
@#endif
@#if (instrument == 9)
    instrumentname = 'tauK';
@#endif
@#if (instrument == 10)
    instrumentname = 'tauR';
@#endif
@#if (instrument == 11)
    instrumentname = 'tauLC';
@#endif

%================================================================
% 2. Defining variables and paramters
%================================================================

% see paper and technical appendix for variable and parameter definitons

% general variables
var Cons        ${C}$
    ConsU       ${C^U}$
    ConsUtilde  ${\tilde{C}^U}$
    ConsC       ${C^C}$
    ConsCtilde  ${\tilde{C}^C}$
    MU_Cons     ${\tilde{\lambda}P^C/(1-\tau^W)}$   % deviating notation
    i           ${i}$
    iW          ${i^W}$
    r           ${r}$
    W           ${W}$
    Wstar       ${W^*}$
    WF          ${W^F}$
    WW          ${W^W}$
    PK          ${P^K}$
    PF          ${P^F}$
    Ph          ${P}$                               % deviating notation
    Phstar      ${P^*}$
    MC          ${MC}$
    x1          ${\Phi^1}$                          % deviating notation
    x2          ${\Phi^2}$                          % deviating notation
    xW1         ${\tilde{\Lambda}^1}$               % deviating notation
    xW2         ${\tilde{\Lambda}^2}$               % deviating notation
    LS          ${\hat{L}}$                         % flipped notation
    LStilde     ${\tilde{L}}$
    LShat       ${L}$                               % flipped notation
    Delta       ${\Delta}$
    Lambda      ${\Lambda}$
    H           ${H}$
    A           ${A}$
    Sav         ${Sav}$
    vp          ${v}$
    vw          ${v^w}$
    TFP         ${\Phi}$                            % deviating notation
    KG          ${K^G}$
    Y           ${Y}$
    Inv         ${I}$
    J           ${J}$
    J_I         ${J_I}$
    J_K         ${J_K}$
    K           ${K}$
    Khat        ${\hat{K}}$
    o           ${o}$
    delta       ${\delta}$
    q           ${q}$
    tauL        ${\tau^L}$
    PB          ${PB}$
    VA          ${\hat{Y}}$                         % deviating notation
    GDP         ${GDP}$
    GDPr        ${GDP^r}$
    E           ${E}$
    TB          ${TB}$
    DF          ${D^F}$
;

% fiscal instruments
var CG          ${C^G}$
    IG          ${I^G}$
    subI        ${sub^I}$
    subL        ${sub^L}$
    tauC        ${\tau^C}$
    tauW        ${\tau^W}$
    tauF        ${\tau^F}$
    tauprof     ${\tau^{prof}}$
    tauK        ${\tau^K}$
    tauR        ${\tau^R}$
    tauLC       ${\tau^{LC}}$
;

% price block
var PCh PCm PC Consm Consh PCtilde PInvh PInvm PInv Invm Invh
    PGh PGm PG CGm CGh PIGh PIGm PIG IGm IGh;

predetermined_variables i iW r K KG DF;

% fiscal policy shocks
varexo e_CG e_IG e_subI e_subL e_tauC e_tauW e_tauF e_tauprof e_tauK e_tauR;

@#if (no_constHH == 0)
    varexo e_tauLC;
@#endif

% general parameters
parameters  alpha        ${\alpha}$
            sigmaP       ${\sigma^P}$
            sigC         ${1/\sigma}$       % inverse elast. of intertemp. substitution
            delta_0      ${\delta_0}$
            delta_1      ${\delta_1}$
            delta_2      ${\delta_2}$
            rho          ${\beta-1}/\beta$  % discount rate
            thetaL       ${\eta}$           % deviating notation
            Psi          ${\psi}$
            sigell       ${1/\sigma^F}$     % inverse Frisch elasticity
            epsilon      ${\epsion}$
            epsilonW     ${\epsilon^w}$
            phi          ${\theta}$         % deviating notation
            phiW         ${\theta^w}$       % deviating notation
            rho_eps      ${\rho^\epsilon}$
            parhabit     ${\kappa}$         % deviating notation
            parhabitL    ${\kappa^L}$       % deviating notation
            FC           ${FC}$
            unconst      ${\pi}$            % deviating notation
            A_0          ${A^0}$
            epsG         ${\sigma^G}$       % deviating notation
            deltaG       ${\delta^G}$
            gam          ${\gamma}$
            xiCm         ${\xi^C}$
            sigCm        ${\lambda^C}$      % deviating notation
            xiIm         ${\xi^I}$
            sigIm        ${\lambda^I}$      % deviating notation
            xiGm         ${\xi^{C^G}}$
            sigGm        ${\lambda^{C^G}}$  % deviating notation
            xiIGm        ${\xi^{I^G}}$
            sigIGm       ${\lambda^{I^G}}$  % deviating notation
            Pm           ${P^m}$
            E_0          ${E^0}$
            sigExport    ${-\lambda^E}$     % deviating notation
            risk         ${-\psi^{DF}}$     % deviating notation
            i0           ${i^*}$            % deviating notation
            DGr0         ${D^{G,r}_0}$       
;

% calibration values of selected variables
parameters  Ph0 PC0 PInv0 PG0 PIG0 DF0 VA0;

% calibration values of fiscal instruments and tax bases
parameters  CG0 IG0 subI0 subL0 tauC0 tauF0 tauK0 tauLC0 tauR0 tauW0
            tauprof0 PB0 TaxCbase0 TaxWbase0 TaxFbase0 Taxprofbase0
            TaxKbase0 TaxRbase0 Inv0;

%================================================================
% 3. Calibration
%================================================================

gam             = param_gam;
alpha           = param_alpha;
sigC            = param_sigC;
delta_0         = param_delta_0;
delta_1         = param_delta_1;
delta_2         = param_delta_2;
rho             = param_rho;
thetaL          = param_thetaL;
Psi             = param_Psi;
sigell          = param_sigell;
epsilon         = param_epsilon;
epsilonW        = param_epsilonW;
phi             = param_phi;
phiW            = param_phiW;
FC              = param_FC0;
parhabit        = param_parhabit;
parhabitL       = param_parhabitL;
deltaG          = param_deltaG;
A_0             = param_A_0;
epsG            = param_epsG;
sigmaP          = param_sigmaP;

@#if (no_constHH == 0)
    unconst         = param_unconst;
@#else
    unconst         = 1.0;
@#endif

subI0           = ss_subI0;
subL0           = ss_subL0;
tauC0           = ss_tauC0;
tauF0           = ss_tauF0;
tauK0           = ss_tauK0;
tauLC0          = ss_tauLC0;
tauR0           = ss_tauR0;
tauW0           = ss_tauW0;
tauprof0        = ss_tauprof0;
PB0             = ss_PB0;
DGr0            = ss_DGr0;
CG0             = ss_CG0;
IG0             = ss_IG0;

TaxCbase0       = ss_TaxCbase0;
TaxWbase0       = ss_TaxWbase0;
TaxFbase0       = ss_TaxFbase0;
Taxprofbase0    = ss_Taxprofbase0;
TaxKbase0       = ss_TaxKbase0;
TaxRbase0       = ss_TaxRbase0;
Inv0            = ss_Inv0;

Ph0             = ss_Ph0;
Pm              = ss_Pm0;
PC0             = ss_PC0;
PG0             = ss_PG0;
PInv0           = ss_PInv0;
PIG0            = ss_PIG0;

xiCm            = param_xiCm;
sigCm           = param_sigCm;
xiIm            = param_xiIm;
sigIm           = param_sigIm;
xiGm            = param_xiGm;
sigGm           = param_sigGm;
xiIGm           = param_xiIGm;
sigIGm          = param_sigIGm;

DF0             = ss_DF0;
VA0             = ss_VA0;
E_0             = param_E_0;
sigExport       = param_sigExport;
risk            = param_risk;
i0              = ss_i0;

rho_eps         = 0.7;

%================================================================
% 4. Model block
%================================================================

model; 
  
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % Parameter short cuts
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    #beta = 1/(1+rho);
    #sig  = 1/sigC;
    #rhoP = (sigmaP-1)/sigmaP;

    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % After-tax prices and real interest
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    iW(+1) = (1-tauR)*i(+1);
    (1+r(+1)) = (1+i(+1))*Ph/Ph(+1);
    PF = (1+tauK)*PK;
    WF = (1+tauF)*W;
    WW = (1-tauW)*W;
    
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % Price block and demand split
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    % ------ private consumption Cons ------
    PCh     = Ph*(1+tauC);
    PCm     = Pm*(1+tauC);
    @#if (CD_Cons == 1)
        PC      = PCm^xiCm*PCh^(1-xiCm);
        PCtilde = Pm^xiCm*Ph^(1-xiCm);
    @#else
        PC      = (xiCm*PCm^(1-sigCm)+(1-xiCm)*PCh^(1-sigCm))^(1/(1-sigCm));
        PCtilde = (xiCm*Pm^(1-sigCm)+(1-xiCm)*Ph^(1-sigCm))^(1/(1-sigCm));
    @#endif
    Consm   = (xiCm*(PC/PCm)^sigCm)*Cons;
    Consh   = ((1-xiCm)*(PC/PCh)^sigCm)*Cons;

    % ------ private investment Inv ------
    PInvh     = Ph;
    PInvm     = Pm;
    @#if (CD_Inv == 1)
        PInv      = PInvm^xiIm*PInvh^(1-xiIm);
    @#else
        PInv      =  (xiIm*PInvm^(1-sigIm)+(1-xiIm)*PInvh^(1-sigIm))^(1/(1-sigIm));
    @#endif
    Invm      = (xiIm*(PInv/PInvm)^sigIm)*(Inv+J);
    Invh      = ((1-xiIm)*(PInv/PInvh)^sigIm)*(Inv+J);

    % ------ public consumption CG ------
    PGh     = Ph;
    PGm     = Pm;
    @#if (CD_CG == 1)
        PG      =  PGm^xiGm*PGh^(1-xiGm);
    @#else
        PG      =  (xiGm*PGm^(1-sigGm)+(1-xiGm)*PGh^(1-sigGm))^(1/(1-sigGm));
    @#endif
    CGm     = (xiGm*(PG/PGm)^sigGm)*CG;
    CGh     = ((1-xiGm)*(PG/PGh)^sigGm)*CG;

    % ------ public investment IG ------
    PIGh    = Ph;
    PIGm    = Pm;
    @#if (CD_IG == 1)
        PIG      =  PIGm^xiIGm*PIGh^(1-xiIGm);
    @#else
        PIG      =  (xiIGm*PIGm^(1-sigIGm)+(1-xiIGm)*PIGh^(1-sigIGm))^(1/(1-sigIGm));
    @#endif
    IGm     = (xiIGm*(PIG/PIGm)^sigIGm)*IG;
    IGh     = ((1-xiIGm)*(PIG/PIGh)^sigIGm)*IG;

    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % Consumption block
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    PC*ConsC = WW*LS-Ph*(tauL+tauLC);
    ConsCtilde = ConsC-parhabit*gam*ConsC(-1);
    ConsUtilde = ConsU-parhabit*gam*ConsU(-1);
    Cons = unconst*ConsU+(1-unconst)*ConsC;
    MU_Cons = (unconst*ConsUtilde + (1-unconst)*ConsCtilde)^(-sigC);
    Delta = PC + parhabit*gam/(1+iW(+1))*Delta(+1);
    Lambda = Delta*(PC)^(-sig)+beta^sig*(1+iW(+1))^(sig-1)*gam*Lambda(+1); 
    H = WW*LS-Ph*tauL+gam*H(+1)/(1+iW(+1));
    Sav = A+WW*LS-PC*Cons-Ph*(tauL+(1-unconst)*tauLC);  
    A(+1) = (1+iW(+1))*Sav;
    ConsU = (A/unconst+H-Delta*parhabit*gam*ConsU(-1))/(Lambda*(PC^sig))+parhabit*gam*ConsU(-1);
    %ConsUtilde(+1) + (1-gam)/gam/(Lambda(+1)*(PC(+1)^sig))*(A(+1)/unconst-Delta(+1)*parhabit*gam*ConsU) = ((beta*(1+iW(+1))*PC/PC(+1))^sig)*ConsUtilde;

    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % Captial accumulation
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    @#if (finite_delta_2 == 1)
        delta = delta_0+delta_1*(o-1)+delta_2/2*(o-1)^2;
        PK = q(+1)/(1+i(+1))*(delta_1+delta_2*(o-1))/(1-tauprof);
    @#else
        delta = delta_0;
        o = 1.0;
    @#endif

    J = Psi/2*K*(Inv/K-delta_0)^2;
    J_I = Psi*(Inv/K-delta_0);
    J_K = Psi/2*(delta_0^2-(Inv/K)^2);
    Khat = K*o;
    K(+1) = (1-delta)*K+Inv;
    q(+1) = (1+i(+1))*PInv*(1-subI+J_I);
    q = (1-tauprof)*PK*o - PInv*J_K + PInv*delta_0*tauprof + q(+1)/(1+i(+1))*(1-delta);
    
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % Intermediate production
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    TFP = A_0*KG^epsG;

    @#if (CD_prod == 1)
        Y = TFP*(Khat^alpha)*(LS^(1-alpha))/vp;
        WF/PF = (1-alpha)/alpha*Khat/LS;
        MC = ((WF/(1-alpha))^(1-alpha)*(PF/alpha)^alpha)/TFP;
    @#else
        Y = TFP*(((alpha^(1-rhoP))*(Khat^rhoP)+((1-alpha)^(1-rhoP))*(LS^rhoP))^(1/rhoP))/vp;
        (WF/PF)^sigmaP = (1-alpha)/alpha*Khat/LS;
        MC = ((alpha*(PF^(1-sigmaP))+(1-alpha)*(WF^(1-sigmaP)))^(1/(1-sigmaP)))/TFP;
    @#endif

    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % Price setting
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    @#if (finite_epsilon == 1)
        vp = (1-phi)*((Phstar/Ph)^(-epsilon))+phi*((Ph(-1)/Ph)^(-epsilon))*vp(-1);
        Ph^(1-epsilon) = (1-phi)*(Phstar^(1-epsilon))+phi*(Ph(-1)^(1-epsilon));
        Phstar = epsilon/(epsilon-1)*x1/x2;
        x1 = MC*Y*Ph^epsilon+phi*x1(+1)/(1+i(+1));
        x2 = Y*Ph^epsilon+phi*x2(+1)/(1+i(+1));
    @#else
        vp = 1.0;
        Ph = MC;
        Phstar = x1/x2;
        x1 = Ph;
        x2 = 1.0;
    @#endif

    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % Wage setting and labor supply
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    @#if (finite_epsilonW == 1)
        vw = (1-phiW)*((Wstar/W)^(-epsilonW))+phiW*((W(-1)/W)^(-epsilonW))*vw(-1);
        W^(1-epsilonW) = (1-phiW)*Wstar^(1-epsilonW)+phiW*(W(-1)^(1-epsilonW));
        Wstar = epsilonW/(epsilonW-1)*xW1/xW2;
        xW1 = thetaL*W^epsilonW*LS*LStilde^sigell+phiW*beta*xW1(+1);
        xW2 = (1-tauW)*MU_Cons/PC*W^epsilonW*LS+phiW*beta*xW2(+1);
        LStilde = ((W/Wstar)^epsilonW)*LS-parhabitL*LShat(-1);
        LShat = vw*LS;
    @#else
        vw = 1.0;
        W = Wstar;
        Wstar = xW1/xW2;
        xW1 = thetaL*LStilde^sigell;
        xW2 = (1-tauW)*MU_Cons/PC;
        LStilde = LS-parhabitL*LShat(-1);
        LShat = vw*LS;
    @#endif

    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % Foreign trade block
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Ph = exp((E+sigExport*log(E_0))/sigExport);
    TB = Ph*E-Pm*(Consm+Invm+CGm+IGm);
    DF(+1) = (1+i(+1))*(DF+TB);
    i(+1) = i0*exp(-risk*(DF/VA-DF0/VA0));

    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % Goods market clearing, value added and GDP
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Cons + Inv + J + CG + IG + TB = Y - FC;
    VA = Ph*(Y-FC);
    GDP = PC*Cons+PInv*(Inv+J)+PG*CG+PIG*IG+TB;
    GDPr = PC0*Cons+PInv0*(Inv+J)+PG0*CG+PIG0*IG+Ph0*E-Pm*(Consm+Invm+CGm+IGm);
    
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % Government
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    PB = tauC*PCtilde*Cons + (tauF+tauW)*W*LS + tauK*PK*Khat + tauprof*(Ph*(Y-FC)-PInv*delta_0*K-tauK*PK*Khat-WF*LS) + tauR*i(+1)/(1+i(+1))*Sav + Ph*(1-unconst)*tauLC - PG*CG - PIG*IG - subI*PInv*Inv - subL*Ph;
    PB + Ph*tauL = r(+1)/(1+r(+1))*DGr0*Ph/Ph0;
    KG(+1) = (1-deltaG)*KG+IG;

    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    % Fiscal instruments
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    CG = (1-rho_eps)*CG0+rho_eps*CG(-1)+e_CG/PG0; 
    IG = (1-rho_eps)*IG0+rho_eps*IG(-1)+e_IG/PIG0;
    subI = (1-rho_eps)*subI0+rho_eps*subI(-1)+e_subI/(PInv0*Inv0);
    subL = (1-rho_eps)*subL0+rho_eps*subL(-1)+e_subL/Ph0;
    tauC = (1-rho_eps)*tauC0+rho_eps*tauC(-1)-e_tauC/TaxCbase0;
    tauW = (1-rho_eps)*tauW0+rho_eps*tauW(-1)-e_tauW/TaxWbase0;
    tauF = (1-rho_eps)*tauF0+rho_eps*tauF(-1)-e_tauF/TaxFbase0;
    tauprof = (1-rho_eps)*tauprof0+rho_eps*tauprof(-1)-e_tauprof/Taxprofbase0;
    tauK = (1-rho_eps)*tauK0+rho_eps*tauK(-1)-e_tauK/TaxKbase0;
    tauR = (1-rho_eps)*tauR0+rho_eps*tauR(-1)-e_tauR/TaxRbase0;
    
    @#if (no_constHH == 0)
        tauLC = (1-rho_eps)*tauLC0+rho_eps*tauLC(-1)-e_tauLC/(Ph0*(1-unconst));
    @#else
        tauLC = 0.0;
    @#endif

end;

%================================================================
% 5. Simulation
%================================================================

initval;
        r            = param_r0;
        i            = ss_i0;
        iW           = ss_iW0;
        Ph           = ss_Ph0;
        PC           = ss_PC0;
        W            = ss_W0;
        Wstar        = ss_Wstar0;
        WF           = ss_WF0;
        WW           = ss_WW0;
        LS           = ss_LS0;
        LStilde      = ss_LStilde0;
        LShat        = ss_LShat0;
        Delta        = ss_Delta0;
        Lambda       = ss_Lambda0;
        H            = ss_H0;
        ConsU        = ss_ConsU0;
        ConsUtilde   = ss_ConsUtilde0;
        ConsC        = ss_ConsC0;
        ConsCtilde   = ss_ConsCtilde0;
        Cons         = ss_Cons0;
        MU_Cons      = ss_MU_Cons0;
        MC           = ss_MC0;
        Y            = ss_Y0;
        vp           = ss_vp0;
        vw           = ss_vw0;
        Phstar       = ss_Phstar0;
        x1           = ss_x10;
        x2           = ss_x20;
        xW1          = ss_xW10;
        xW2          = ss_xW20;
        KG           = ss_KG0;
        TFP          = ss_TFP0;
        Inv          = ss_Inv0;
        K            = ss_K0;
        Khat         = ss_Khat0;
        o            = ss_o0;
        delta        = ss_delta0;
        J            = 0;
        J_I          = 0;
        J_K          = 0;
        PK           = ss_PK0;
        PF           = ss_PF0;
        q            = ss_q0;
        tauL         = ss_tauL0;
        A            = ss_A0;
        Sav          = ss_Sav0;
        DF           = ss_DF0;
        E            = ss_E0;
        TB           = ss_TB0;
        VA           = ss_VA0;
        PB           = ss_PB0;
        GDP          = ss_GDP0;
        GDPr         = ss_GDPr0;

        CG           = ss_CG0;
        IG           = ss_IG0;
        subI         = ss_subI0;
        subL         = ss_subL0;
        tauC         = ss_tauC0;
        tauW         = ss_tauW0;
        tauF         = ss_tauF0;
        tauprof      = ss_tauprof0;
        tauK         = ss_tauK0;
        tauR         = ss_tauR0;
        tauLC        = ss_tauLC0;
        
        PC           = ss_PC0;
        PCm          = ss_PCm0;  
        PCh          = ss_PCh0;
        Consm        = ss_Consm0;
        Consh        = ss_Consh0;
        PCtilde      = ss_PCtilde0;

        PInv         = ss_PInv0;
        PInvm        = ss_PInvm0;
        PInvh        = ss_PInvh0;
        Invh         = ss_Invh0;
        Invm         = ss_Invm0;

        PG           = ss_PG0;
        PGm          = ss_PGm0;
        PGh          = ss_PGh0;
        CGm          = ss_CGm0;
        CGh          = ss_CGh0;

        PIG          = ss_PIG0;
        PIGh         = ss_PIGh0;
        PIGm         = ss_PIGm0;
        IGm          = ss_IGm0;
        IGh          = ss_IGh0;

end;

steady;
check;

%return;

@#if (shocktype == 1)

    endval;
        r            = param_r0;
        i            = ss_i0;
        iW           = ss_iW0;
        Ph           = ss_Ph0;
        PC           = ss_PC0;
        W            = ss_W0;
        Wstar        = ss_Wstar0;
        WF           = ss_WF0;
        WW           = ss_WW0;
        LS           = ss_LS0;
        LStilde      = ss_LStilde0;
        LShat        = ss_LShat0;
        Delta        = ss_Delta0;
        Lambda       = ss_Lambda0;
        H            = ss_H0;
        ConsU        = ss_ConsU0;
        ConsUtilde   = ss_ConsUtilde0;
        ConsC        = ss_ConsC0;
        ConsCtilde   = ss_ConsCtilde0;
        Cons         = ss_Cons0;
        MU_Cons      = ss_MU_Cons0;
        MC           = ss_MC0;
        Y            = ss_Y0;
        vp           = ss_vp0;
        vw           = ss_vw0;
        Phstar       = ss_Phstar0;
        x1           = ss_x10;
        x2           = ss_x20;
        xW1          = ss_xW10;
        xW2          = ss_xW20;
        KG           = ss_KG0;
        TFP          = ss_TFP0;
        Inv          = ss_Inv0;
        K            = ss_K0;
        Khat         = ss_Khat0;
        o            = ss_o0;
        delta        = ss_delta0;
        J            = 0;
        J_I          = 0;
        J_K          = 0;
        PK           = ss_PK0;
        PF           = ss_PF0;
        q            = ss_q0;
        tauL         = ss_tauL0;
        A            = ss_A0;
        Sav          = ss_Sav0;
        DF           = ss_DF0;
        E            = ss_E0;
        TB           = ss_TB0;
        VA           = ss_VA0;
        PB           = ss_PB0;
        GDP          = ss_GDP0;
        GDPr         = ss_GDPr0;

        CG           = ss_CG0;
        IG           = ss_IG0;
        subI         = ss_subI0;
        subL         = ss_subL0;
        tauC         = ss_tauC0;
        tauW         = ss_tauW0;
        tauF         = ss_tauF0;
        tauprof      = ss_tauprof0;
        tauK         = ss_tauK0;
        tauR         = ss_tauR0;
        tauLC        = ss_tauLC0;
        
        PC           = ss_PC0;
        PCm          = ss_PCm0;  
        PCh          = ss_PCh0;
        Consm        = ss_Consm0;
        Consh        = ss_Consh0;
        PCtilde      = ss_PCtilde0;

        PInv         = ss_PInv0;
        PInvm        = ss_PInvm0;
        PInvh        = ss_PInvh0;
        Invh         = ss_Invh0;
        Invm         = ss_Invm0;

        PG           = ss_PG0;
        PGm          = ss_PGm0;
        PGh          = ss_PGh0;
        CGm          = ss_CGm0;
        CGh          = ss_CGh0;

        PIG          = ss_PIG0;
        PIGh         = ss_PIGh0;
        PIGm         = ss_PIGm0;
        IGm          = ss_IGm0;
        IGh          = ss_IGh0;

        @#if (instrument == 1)
        e_CG = shock_size;
        @#endif
        @#if (instrument == 2)
        e_IG = shock_size;
        @#endif
        @#if (instrument == 3)
        e_subI = shock_size;
        @#endif
        @#if (instrument == 4)
        e_subL = shock_size;
        @#endif
        @#if (instrument == 5)
        e_tauC = shock_size;
        @#endif
        @#if (instrument == 6)
        e_tauW = shock_size;
        @#endif
        @#if (instrument == 7)
        e_tauF = shock_size;
        @#endif
        @#if (instrument == 8)
        e_tauprof = shock_size;
        @#endif
        @#if (instrument == 9)
        e_tauK = shock_size;
        @#endif
        @#if (instrument == 10)
        e_tauR = shock_size;
        @#endif
        @#if (instrument == 11)
        e_tauLC = shock_size;
        @#endif

    end;

    rho_eps         = 0.0;

    steady;

    simul(periods=500,tolf=1e-10,tolx=1e-10);

@#endif
@#if (shocktype == 2)
    
    shocks;
    
    @#if (instrument == 1)
      var e_CG;
    @#endif
    @#if (instrument == 2)
      var e_IG;
    @#endif
    @#if (instrument == 3)
      var e_subI;
    @#endif
    @#if (instrument == 4)
      var e_subL;
    @#endif
    @#if (instrument == 5)
      var e_tauC;
    @#endif
    @#if (instrument == 6)
      var e_tauW;
    @#endif 
    @#if (instrument == 7)
      var e_tauF;
    @#endif 
    @#if (instrument == 8)
      var e_tauprof;
    @#endif 
    @#if (instrument == 9)
      var e_tauK;
    @#endif 
    @#if (instrument == 10)
      var e_tauR;
    @#endif
    @#if (instrument == 11)
      var e_tauLC;
    @#endif 

    periods 1;
    values (shock_size);
    
    end;

    simul(periods=100,tolf=1e-10,tolx=1e-10);

@#endif
@#if (shocktype == 3)

    shocks;
    
    @#if (instrument == 1)
      var e_CG = shock_size^2;
    @#endif
    @#if (instrument == 2)
      var e_IG = shock_size^2;
    @#endif
    @#if (instrument == 3)
      var e_subI = shock_size^2;
    @#endif
    @#if (instrument == 4)
      var e_subL = shock_size^2;
    @#endif
    @#if (instrument == 5)
      var e_tauC = shock_size^2;
    @#endif
    @#if (instrument == 6)
      var e_tauW = shock_size^2;
    @#endif
    @#if (instrument == 7)
      var e_tauF = shock_size^2;
    @#endif
    @#if (instrument == 8)
      var e_tauprof = shock_size^2;
    @#endif
    @#if (instrument == 9)
      var e_tauK = shock_size^2;
    @#endif
    @#if (instrument == 10)
      var e_tauR = shock_size^2;
    @#endif
    @#if (instrument == 11)
      var e_tauLC = shock_size^2;
    @#endif

    end;

    stoch_simul(order = 2, irf=40) VA Y GDPr Cons ConsU ConsC K o Ph Phstar W i r LS IG Inv PK MC PB;

@#endif

%================================================================
% 6. Compute multipliers
%================================================================

if ((shocktype == 1)||(shocktype == 2))
    
    % Note: as FC is constant real absolute change in VA is Ph0*(Y-Y0).
    
    Y0 = ss_Y0;
    GDPr0 = ss_GDPr0;
    tend = length(Y);
    
    if (shocktype == 1)
        shock = [0; ones(tend-1,1)]*shock_size;
    else
        shock = [0; 1; cumprod(ones(tend-2,1)*rho_eps)]*shock_size;
    end;

    expostshock = PB0-PB./Ph*Ph0;
    
    discfactor = cumprod(1./(1+[0;0; r(2:end-1)]));

    % ex-ante multipliers    
    mult = Ph0*(Y-Y0)./shock;
    PVmult = cumsum(Ph0*(Y-Y0).*discfactor)./cumsum(shock.*discfactor);
    PVmult = PVmult(2:end);
    PVmultGDPr = cumsum((GDPr-GDPr0).*discfactor)./cumsum(shock.*discfactor);
    PVmultGDPr = PVmultGDPr(2:end);
    
    % ex-post multipliers    
    mult_expost = Ph0*(Y-Y0)./expostshock;
    PVmult_expost = cumsum(Ph0*(Y-Y0).*discfactor)./cumsum(expostshock.*discfactor);
    PVmult_expost = PVmult_expost(2:end);
    PVmultGDPr_expost = cumsum((GDPr-GDPr0).*discfactor)./cumsum(expostshock.*discfactor);
    PVmultGDPr_expost = PVmultGDPr_expost(2:end);

else
    
    dY    = eval(strcat('Y_e_',instrumentname));
    dGDPr = eval(strcat('GDPr_e_',instrumentname));
    tend  = length(dY);
    shock = [1; cumprod(ones(tend-1,1)*rho_eps)]*shock_size;
    dPh    = eval(strcat('Ph_e_',instrumentname));
    dPB    = eval(strcat('PB_e_',instrumentname));
    expostshock = PB0-(PB0+dPB)./(Ph0+dPh)*Ph0;
    dr    = eval(strcat('r_e_',instrumentname));
        
    r0 = param_r0;    
    r2 = r0+dr;
    discfactor = cumprod(1./(1+[0; r2(2:end)]));
    
    % ex-ante multipliers    
    mult = Ph0*dY./shock;
    PVmult = cumsum(Ph0*dY.*discfactor)./cumsum(shock.*discfactor);
    PVmultGDPr = cumsum(dGDPr.*discfactor)./cumsum(shock.*discfactor);

    % ex-post multipliers    
    mult_expost = Ph0*dY./expostshock;
    PVmult_expost = cumsum(Ph0*dY.*discfactor)./cumsum(expostshock.*discfactor);
    PVmultGDPr_expost = cumsum(dGDPr.*discfactor)./cumsum(expostshock.*discfactor);

end;

figure(10);
hold on;
Plot1 = plot(PVmult(1:40),'color','blue');          Line1 = 'Value added PV multiplier';
Plot2 = plot(PVmultGDPr(1:40),'color','red');      Line2 = 'real GDP PV multiplier';
Plot3 = plot(PVmult_expost(1:40),'color','black');   Line3 = 'Value added PV multiplier (ex-post)';
legend([Plot1;Plot2;Plot3], Line1, Line2, Line3);


