% A New Keynesian model in a small open economy
% Replicates the model from "On fiscal multipliers in New Keynesian small 
%   open economy models" by Philip Schuster
% Philip Schuster Nov 14, 2019, philip.schuster@oenb.at

function[] = calibration()

global param ss

attach_struct(param);
attach_struct(ss);

i0       = r0;
iW0      = i0*(1-tauR0);
DG0      = Ph0*DGrealshare*GDP0;
DGr0     = DG0*Ph0/Ph0;
sigC     = 1/sig;
tauC0    = 1/(Consshare/TaxCshare*GDP0-1);

%#############################################
% PRICE AND UNIT DEMAND BLOCK

PCh0     =  Ph0*(1+tauC0);
PCm0     =  Pm0*(1+tauC0);
if (sigCm == 1) 
  PC0      = PCm0^xiCm*PCh0^(1-xiCm);
  PCtilde0 = Pm0^xiCm*Ph0^(1-xiCm);
else
  PC0      =  (xiCm*PCm0^(1-sigCm)+(1-xiCm)*PCh0^(1-sigCm))^(1/(1-sigCm));
  PCtilde0 =  (xiCm*Pm0^(1-sigCm)+(1-xiCm)*Ph0^(1-sigCm))^(1/(1-sigCm)); % such that PC0 = PCtilde0*(1+tauC0)
end
consm0   =  xiCm*(PC0/PCm0)^sigCm;
consh0   =  (1-xiCm)*(PC0/PCh0)^sigCm;

PInvh0     =  Ph0;
PInvm0     =  Pm0; % = 1, Pm = Pm0 for all t
if (sigIm == 1)
  PInv0      = PInvm0^xiIm*PInvh0^(1-xiIm);
else
  PInv0      =  (xiIm*PInvm0^(1-sigIm)+(1-xiIm)*PInvh0^(1-sigIm))^(1/(1-sigIm));
end
invm0      =  xiIm*(PInv0/PInvm0)^sigIm;
invh0      =  (1-xiIm)*(PInv0/PInvh0)^sigIm;

PGh0     =  Ph0;
PGm0     =  Pm0; % = 1, Pm = Pm0 for all t
if (sigGm == 1)
  PG0      =  PGm0^xiGm*PGh0^(1-xiGm);
else
  PG0      =  (xiGm*PGm0^(1-sigGm)+(1-xiGm)*PGh0^(1-sigGm))^(1/(1-sigGm));
end
cgm0     =  xiGm*(PG0/PGm0)^sigGm;
cgh0     =  (1-xiGm)*(PG0/PGh0)^sigGm;

PIGh0     =  Ph0;
PIGm0     =  Pm0; % = 1, Pm = Pm0 for all t
if (sigIGm == 1)
  PIG0      =  PIGm0^xiIGm*PIGh0^(1-xiIGm);
else
  PIG0      =  (xiIGm*PIGm0^(1-sigIGm)+(1-xiIGm)*PIGh0^(1-sigIGm))^(1/(1-sigIGm));
end
invGm0    =  xiIGm*(PIG0/PIGm0)^sigIGm;
invGh0    =  (1-xiIGm)*(PIG0/PIGh0)^sigIGm;
%#############################################

Cons0   = Consshare*GDP0/PC0;
Inv0    = Invshare*GDP0/PInv0;
CG0     = CGshare*GDP0/PG0;
IG0     = IGshare*GDP0/PIG0;

Consh0  = consh0*Cons0;
Consm0  = consm0*Cons0;
CGh0    = cgh0*CG0;
CGm0    = cgm0*CG0;
IGh0    = invGh0*IG0;
IGm0    = invGm0*IG0;

TaxC0   = TaxCshare*GDP0;
VA0     = GDP0-TaxC0;

% market power
if (isfinite(epsilon))
  markup0     = epsilon/(epsilon-1);
  Y0          = VA0/(Ph0*(1-fixedcostsshare/epsilon));
  FC0         = Y0/epsilon*fixedcostsshare; % CAREFUL: this works only in this calibration strategy
else
  markup0     = 1.0;
  Y0          = VA0/Ph0;
  FC0         = 0.0;
end

if (isfinite(epsilonW))
  markupW0    = epsilonW/(epsilonW-1);
else
  markupW0    = 1.0;
end

MC0      = Ph0/markup0;
WF0      = W0*(1+tauF0);
Wstar0   = W0;
WW0      = W0*(1-tauW0);
LD0      = laborshare*GDP0/WF0;    % just marco matching (profits are included in GOS)
LS0      = LD0;

if (isfinite(epsilonW))
  vw0      = (1-phiW)*(Wstar0/W0)^(-epsilonW)/(1-phiW*(W0/W0)^(-epsilonW));
else
  vw0      = 1.0;
end

LShat0   = vw0*LS0;
LStilde0 = LS0-parhabitL*LShat0;

if (isfinite(epsilon))
  x10      = MC0*Y0*Ph0^(epsilon)/(1-phi/(1+i0));
  x20      = Y0*Ph0^(epsilon)/(1-phi/(1+i0));
  Phstar0  = markup0*x10/x20;
  vp0      = (1-phi)*(Phstar0/Ph0)^(-epsilon)/(1-phi*(Ph0/Ph0)^(-epsilon));
else
  x10      = Ph0;
  x20      = 1.0;
  Phstar0  = markup0*x10/x20;
  vp0      = 1.0;
end

ss.i0          = i0;
ss.iW0         = iW0;
ss.WF0         = WF0;
ss.WW0         = WW0;
ss.Wstar0      = Wstar0;
ss.PC0         = PC0;
ss.PCtilde0    = PCtilde0;
ss.PCm0        = PCm0;
ss.PCh0        = PCh0;
ss.PInv0       = PInv0;
ss.PInvm0      = PInvm0;
ss.PInvh0      = PInvh0;
ss.PG0         = PG0;
ss.PGm0        = PGm0;
ss.PGh0        = PGh0;
ss.PIG0        = PIG0;
ss.PIGm0       = PIGm0;
ss.PIGh0       = PIGh0;
ss.x10         = x10;
ss.x20         = x20;
ss.Phstar0     = Phstar0;
ss.vp0         = vp0;
ss.vw0         = vw0;
ss.Inv0        = Inv0;
ss.Cons0       = Cons0;
ss.Consh0      = Consh0;
ss.Consm0      = Consm0;
ss.CG0         = CG0;
ss.CGh0        = CGh0;
ss.CGm0        = CGm0;
ss.IG0         = IG0;
ss.IGh0        = IGh0;
ss.IGm0        = IGm0;
ss.Y0          = Y0;
ss.VA0         = VA0;
ss.MC0         = MC0;
ss.markup0     = markup0;
ss.markupW0    = markupW0;
ss.LS0         = LS0;
ss.LStilde0    = LStilde0;
ss.LShat0      = LShat0;
ss.LD0         = LD0;
ss.DG0         = DG0;
ss.DGr0        = DGr0;
ss.TaxC0       = TaxC0;
ss.tauC0       = tauC0;

param.FC0      = FC0;
param.sigC     = sigC;

[xout, feval, check] = fsolve(@calibfind_delta,0.01,optimoptions('fsolve','Display','none'));
if (check ~=1); error('No roots found for calibfind_delta!'); end

attach_struct(ss);

KG0      = IG0/deltaG;

if (KG0 == 0)
  A_0    = TFP0;
else
  A_0    = TFP0/(KG0^epsG);
end

Invh0   = invh0*(Inv0+J0);
Invm0   = invm0*(Inv0+J0);

ss.KG0        = KG0;
ss.Invh0      = Invh0;
ss.Invm0      = Invm0;
param.A_0     = A_0;

[xout, feval, check] = fsolve(@calibfind,[0.01,i0*(1-tauR0),0],optimoptions('fsolve','Display','none'));
if (check ~=1); error('No roots found for calibfind!'); end

attach_struct(param)
attach_struct(ss)

Wstarcheck = markupW0*xW10/xW20-Wstar0;
ConsUcheck = ConsU0-ConsUtemp;
GDPcheck   = Ph0*(Y0-FC0)+TaxC0-GDP0;

fprintf('CHECK CALIBRATION: %1.5f\n', sum(abs([eda0,edy0,edl0,edg0,edv0,Wstarcheck,ConsUcheck,GDPcheck]))); 

end