% A New Keynesian model in a small open economy
% Replicates the model from "On fiscal multipliers in New Keynesian small 
%   open economy models" by Philip Schuster
% Philip Schuster May 28, 2020, philip.schuster@oenb.at

clear all;
close all;

fprintf('\nA NEW KEYSIAN SMALL OPEN ECONOMY MODEL\n\n'); 

%================================================================
% 1. Set parameters (stored in structures 'param' and 'ss')
%================================================================

global param ss

% Note: if useRBC == 1, set finite_epsilon, finite_epsilonW, finite_delta_2
% all to 0 and no_unconstHH to 1 in section 1.a in NK_soe.mod
useRBC                    = 0;                  % 0

param.r0                  = 0.03/4;             % 0.03/4
param.gam                 = 160/161;            % 160/161
param.delta_2             = 0.05;               % 0.05
param.deltaG              = 0.0125;             % 0.0125
param.epsG                = 0.08;               % 0.08
param.sig                 = 0.7;                % 0.7 
param.sigell              = 1.0;                % 1.0
param.sigmaP              = 1.0;                % 1.0
param.phi                 = 0.7;                % 0.7
param.phiW                = 0.5;                % 0.5
param.epsilon             = 11.0;               % 11.0
param.epsilonW            = 11.0;               % 11.0
param.Psi                 = 10;                 % 10.0
param.risk                = 0.15;               % 0.15
param.parhabit            = 0.5;                % 0.5
param.parhabitL           = 0.25;               % 0.25
param.fixedcostsshare     = 0.1;                % 0.1
param.unconst             = 0.7;                % 0.7
param.sigCm               = 1.2;                % 1.2
param.sigGm               = 0.8;                % 0.8
param.sigIm               = 1.2;                % 1.2
param.sigIGm              = 0.8;                % 0.8
param.xiCm                = 0.277;              % 0.277
param.xiGm                = 0.116;              % 0.116
param.xiIm                = 0.371;              % 0.371
param.xiIGm               = 0.116;              % 0.116
param.sigExport           = -0.012/log(1.01);   % -0.012/log(1.01);

% NORMALIZATIONS
ss.GDP0                   = 1.0;                % 1.0
ss.Ph0                    = 1.0;                % 1.0
ss.Pm0                    = 1.0;                % 1.0
ss.W0                     = 1.0;                % 1.0
ss.o0                     = 1.0;                % 1.0

% MACRO SHARES
ss.laborshare             = 0.551;              % 0.551
ss.Consshare              = 0.567;              % 0.567
ss.Invshare               = 0.186;              % 0.186

% PUBLIC SECTOR
ss.subI0                  = 0.01;               % 0.01
ss.subL0                  = 0.01;               % 0.01
ss.tauF0                  = 0.22;               % 0.22
ss.tauK0                  = 0.015;              % 0.015
ss.tauLC0                 = 0.0;                % 0.0
ss.tauR0                  = 0.1;                % 0.1
ss.tauW0                  = 0.34;               % 0.34
ss.tauprof0               = 0.15;               % 0.15

ss.TaxCshare              = 0.108;              % 0.108
ss.IGshare                = 0.03;               % 0.03
ss.CGshare                = 0.197;              % 0.197
ss.DGrealshare            = 0.7;                % 0.7

if (useRBC == 1)
    param.gam         = 161/161;
    param.delta_2     = Inf;
    param.epsilon     = Inf;
    param.epsilonW    = Inf;
    param.parhabit    = 0.0;
    param.parhabitL   = 0.0;
    param.sig         = 0.9;
    param.sigell      = 1/0.7;
    param.unconst     = 1.0;
end

%================================================================
% 2. Calibration for Austria
%================================================================

calibration();
attach_struct(param,'param_');
attach_struct(ss,'ss_');

%================================================================
% 3. Run model in Dynare
%================================================================

dynare NK_soe.mod noclearall

%================================================================
% README
%================================================================
%{

ABOUT: This is an UNOFFICIAL replication of the base model from "On fiscal 
multipliers in New Keynesian small open economy models" by Philip Schuster
in MATLAB/Dynare.

DISCLAIMER: In contrast to the actual toolbox used for the paper this 
implementation was not thoroughly tested, may contain errors and comes
without warranty. The implementation is limited to the benchmark model 
(one industry, one budget rule [=instantaneous budget rule], one 
calibration, only selected results). The implementation should replicate
(parts of) tables 4.1, 4.2 and 4.5 of the paper. Small deviations in
results are attributed to different orders of numerical approximations and 
other Dynare-specific peculiarities I did not attend to (e.g. Dynare shifts
the output of predetermined variables by one period, which introduces an
inconsistency between the output and how the model is declared in the model
block). If you spot any mistake please contact: philip.schuster@oenb.at.

USAGE: Parameters and switches are set in this file (runNK.m) and the user
control center section (1.a) in NK_soe.mod. Always start the simulation by 
running runNK.m (instead of calling NK_soe.mod directly). The initial
steady state can be robustly calculated in calibration() for various 
parameteric choices. Shock type (permanent vs. temporary) and the choice of
fiscal instrument (1 to 11) are set in section 1.a in NK_soe.mod. Further
switches in the same section are introduced to handle limiting cases.

%}
